@@ -253,7 +253,7 @@ class Agent:
             # Add the agent instance.
             kwargs["agent_env"] = self.agent_env
 
-        start_time = time.time()
+        start_time = time.monotonic()
         try:
             if asyncio.iscoroutinefunction(tool.fn):
                 result = await tool.fn(**kwargs)
@@ -281,7 +281,7 @@ class Agent:
             result = f"({tool_call.name}) {e!s}"
             is_error = True
 
-        end_time = time.time()
+        end_time = time.monotonic()
 
         yield ToolResult(
             tool_call_id=tool_call.id,
