@@ -2,7 +2,7 @@ from collections.abc import AsyncGenerator, Sequence
 from dataclasses import dataclass
 from typing import Unpack
 
-from flexai.message import AIMessage, Message, SystemMessage, TextBlock
+from flexai.message import AIMessage, Message, SystemMessage
 from flexai.tool import Tool
 
 from .client import Client as Client
@@ -22,15 +22,9 @@ class DefaultClient(Client):
         tools: Sequence[Tool] | None = None,
         **input_args: Unpack[PartialAgentRunArgs],
     ) -> AIMessage:
-        # Mock implementation that always returns a blank TextBlock
-        # This serves as a placeholder when no real LLM provider is configured
-        return AIMessage(
-            content=[
-                TextBlock(
-                    text="[DEFAULT CLIENT IS BEING USED]",
-                )
-            ]
-        )
+        # Return an empty AIMessage when no real LLM provider is configured
+        # Empty content signals agent loop termination and prevents storing unnecessary messages
+        return AIMessage(content=[])
 
     async def stream_chat_response(
         self,
@@ -39,12 +33,5 @@ class DefaultClient(Client):
         tools: Sequence[Tool] | None = None,
         **input_args: Unpack[PartialAgentRunArgs],
     ) -> AsyncGenerator[AIMessage, None]:
-        # Mock implementation that always returns a blank TextBlock
-        # Yields a single AIMessage with empty send_message tool call
-        yield AIMessage(
-            content=[
-                TextBlock(
-                    text="[DEFAULT CLIENT IS BEING USED]",
-                )
-            ]
-        )
+        # Return an empty AIMessage when no real LLM provider is configured
+        yield AIMessage(content=[])
