@@ -95,6 +95,8 @@ class AnthropicClient(Client):
             defaults["temperature"] = 1.0
         if "force_tool" not in run_args:
             defaults["force_tool"] = True
+        if run_args.get("thinking_budget") is None:
+            defaults["thinking_budget"] = self.default_thinking_budget
         return defaults
 
     async def get_chat_response(
@@ -125,7 +127,7 @@ class AnthropicClient(Client):
             )
 
         # Send the messages to the model and get the response.
-        start = time.time()
+        start = time.monotonic()
         params = self._get_params(
             messages=messages,
             system=system,
@@ -134,19 +136,21 @@ class AnthropicClient(Client):
             **run_args,
         )
         response = await self.client.messages.create(**params)
-        generation_time = time.time() - start
+        generation_time = time.monotonic() - start
 
-        # Parse out the tool uses from the response.
-        tool_uses = [
-            get_tool_call(message)
-            for message in response.content
-            if isinstance(message, ToolUseBlock)
-        ]
+        # Parse out all content from the response (text and tools).
+        formatted_content_parts = []
 
-        # Get the content to return.
-        content_to_return = tool_uses or "\n".join(
-            [message.text for message in response.content]
-        )
+        for message in response.content:
+            if isinstance(message, ToolUseBlock):
+                # Add tool calls
+                formatted_content_parts.append(get_tool_call(message))
+            # Add text content (assume it has .text attribute)
+            elif hasattr(message, "text") and message.text.strip():
+                formatted_content_parts.append(TextBlock(message.text))
+
+        # Return all content parts (allows text + tool calls together)
+        content_to_return = formatted_content_parts or ""
         usage = get_usage_block(response.usage)
         usage.generation_time = generation_time
         return AIMessage(
@@ -176,7 +180,7 @@ class AnthropicClient(Client):
         usage = Usage()
 
         # Iterate over the response stream.
-        start = time.time()
+        start = time.monotonic()
         params = self._get_params(
             messages=messages,
             system=system,
@@ -229,7 +233,7 @@ class AnthropicClient(Client):
             # The end of the content block.
             elif chunk.type == "message_stop":
                 # Parse the tool from the buffer and convert it to a tool call.
-                usage.generation_time = time.time() - start
+                usage.generation_time = time.monotonic() - start
 
                 # Send the final text message.
                 content = None
@@ -405,13 +409,24 @@ Do not begin your response with ```json or end it with ```.
         )
         content = response.content
         try:
+            # Extract text content from TextBlock objects or use string directly
+            if isinstance(content, list):
+                text_content = ""
+                for part in content:
+                    if isinstance(part, TextBlock):
+                        text_content += part.text
+                content = text_content
             if not isinstance(content, str):
                 raise TypeError("The response is not a string.")
 
             # Try to parse the response as a sanity check.
             model_to_use.model_validate_json(content)
 
-            return response
+            # Return response with string content for structured output
+            return AIMessage(
+                content=content,
+                usage=response.usage,
+            )
 
         except Exception as e:
             # Try again, printing the exception.
@@ -473,6 +488,34 @@ Do not begin your response with ```json or end it with ```.
             "input_schema": input_schema,
         }
 
+    @classmethod
+    def _format_tool_result(
+        cls,
+        result: Any,
+    ):
+        if isinstance(result, TextBlock):
+            return [
+                {
+                    "type": "text",
+                    "text": result.text,
+                }
+            ]
+
+        if isinstance(result, ImageBlock):
+            return [
+                {
+                    "type": "image",
+                    "source": {
+                        "type": "base64",
+                        "media_type": result.mime_type,
+                        "data": base64.b64encode(result.image).decode("utf-8"),
+                    },
+                }
+            ]
+
+        # For anything else, just fallback to trying to turn it into a string
+        return str(result)
+
     @classmethod
     def _format_message_content(
         cls,
@@ -528,7 +571,7 @@ Do not begin your response with ```json or end it with ```.
                 {
                     "type": "tool_result",
                     "tool_use_id": content.tool_call_id,
-                    "content": cls._format_message_content([content.result]),
+                    "content": cls._format_tool_result(content.result),
                     "is_error": content.is_error,
                 }
                 if allow_tool
