@@ -26,7 +26,6 @@ from flexai.llm.exceptions import NoContentException
 from flexai.message import (
     AIMessage,
     DataBlock,
-    GroundingBlock,
     ImageBlock,
     Message,
     MessageContent,
@@ -35,7 +34,6 @@ from flexai.message import (
     ThoughtBlock,
     ToolCall,
     ToolResult,
-    URLContextBlock,
     Usage,
 )
 from flexai.tool import Tool, ToolType
@@ -145,9 +143,6 @@ class GeminiClient(Client):
     # The model to use for the client.
     model: str = os.getenv("GEMINI_MODEL") or "gemini-2.5-pro-preview-06-05"
 
-    # Default thinking budget for LLM calls (0 if disabled, None if we don't care)
-    default_thinking_budget: int | None = None
-
     # Project ID for Vertex AI (required when using Vertex AI or global endpoint)
     project_id: str | None = field(default=os.environ.get("GOOGLE_PROJECT_ID"))
 
@@ -428,6 +423,23 @@ class GeminiClient(Client):
                     text=formatted_result,
                 )
 
+            if isinstance(formatted_result, ImageBlock):
+                return [
+                    {
+                        "functionResponse": {
+                            "id": content.tool_call_id,
+                            "name": name_context[content.tool_call_id],
+                            "response": {
+                                "text": "The result of this tool is an image. The next part will contain this image.",
+                            },
+                        },
+                    },
+                    cls._format_message_content(
+                        formatted_result,
+                        name_context=name_context,
+                    ),
+                ]
+
             try:
                 formatted_result = cls._format_message_content(
                     formatted_result, name_context=name_context
@@ -599,7 +611,7 @@ class GeminiClient(Client):
             stream=False,
             **run_args,
         )
-        start = time.time()
+        start = time.monotonic()
         response_object = await self.client.models.generate_content(
             **params,
         )
@@ -614,7 +626,7 @@ class GeminiClient(Client):
             output_tokens=output_tokens,
             cache_read_tokens=cache_read,
             cache_write_tokens=0,  # Currently not accounted for
-            generation_time=time.time() - start,
+            generation_time=time.monotonic() - start,
         )
 
         # Argument to raise if no content is returned
@@ -638,12 +650,7 @@ class GeminiClient(Client):
             return AIMessage(content=[], usage=usage)
 
         formatted_content_parts: list[
-            ImageBlock
-            | TextBlock
-            | ThoughtBlock
-            | ToolCall
-            | URLContextBlock
-            | GroundingBlock
+            ImageBlock | TextBlock | ThoughtBlock | ToolCall
         ] = []
 
         for part in response_content_parts:
@@ -691,14 +698,12 @@ class GeminiClient(Client):
             stream=True,
             **run_args,
         )
-        start = time.time()
+        start = time.monotonic()
         response_object = await self.client.models.generate_content_stream(
             **params,
         )
         text_buffer = None
-        total_content_list: list[
-            TextBlock | ThoughtBlock | ToolCall | URLContextBlock | GroundingBlock
-        ] = []
+        total_content_list: list[TextBlock | ThoughtBlock | ToolCall] = []
 
         async for chunk in response_object:
             # Handle this case
@@ -728,7 +733,7 @@ class GeminiClient(Client):
                             total_content_list.append(to_yield)
                             yield to_yield
 
-        usage.generation_time = time.time() - start
+        usage.generation_time = time.monotonic() - start
         if text_buffer:
             total_content_list.append(text_buffer)
 
@@ -737,116 +742,6 @@ class GeminiClient(Client):
             usage=usage,
         )
 
-    def _extract_url_context_block(
-        self, url_context_metadata
-    ) -> URLContextBlock | None:
-        """Extract URLContextBlock from URL context metadata.
-
-        Args:
-            url_context_metadata: The URL context metadata from Gemini response.
-
-        Returns:
-            URLContextBlock if metadata is present, None otherwise.
-        """
-        if not url_context_metadata:
-            return None
-
-        # Extract URLs and metadata from the URL context
-        urls_accessed = []
-        metadata_dict = {}
-
-        if hasattr(url_context_metadata, "__iter__"):
-            # Handle list of URL metadata objects
-            for url_meta in url_context_metadata:
-                if hasattr(url_meta[1][0], "retrieved_url"):
-                    urls_accessed.append(url_meta[1][0].retrieved_url)
-                # Store the complete metadata object
-                metadata_dict = {"url_metadata": url_context_metadata}
-        else:
-            # Handle single metadata object or dict
-            if hasattr(url_context_metadata, "retrieved_url"):
-                urls_accessed.append(url_context_metadata.retrieved_url)
-            metadata_dict = {"url_metadata": url_context_metadata}
-
-        return URLContextBlock(urls_accessed=urls_accessed, metadata=metadata_dict)
-
-    def _extract_grounding_block(self, grounding_metadata) -> GroundingBlock | None:
-        """Extract GroundingBlock from Google Search grounding metadata.
-
-        Args:
-            grounding_metadata: The grounding metadata from Gemini response.
-
-        Returns:
-            GroundingBlock if metadata is present, None otherwise.
-        """
-        if not grounding_metadata:
-            return None
-
-        # Extract search queries
-        search_queries = []
-        if getattr(grounding_metadata, "web_search_queries", None):
-            search_queries = list(grounding_metadata.web_search_queries or [])
-
-        # Extract grounding chunks
-        grounding_chunks = []
-        if getattr(grounding_metadata, "grounding_chunks", None):
-            grounding_chunks = [
-                {
-                    "web": {
-                        "uri": chunk.web.uri
-                        if hasattr(chunk, "web") and hasattr(chunk.web, "uri")
-                        else "",
-                        "title": chunk.web.title
-                        if hasattr(chunk, "web") and hasattr(chunk.web, "title")
-                        else "",
-                    }
-                }
-                for chunk in grounding_metadata.grounding_chunks
-                if hasattr(chunk, "web")
-            ]
-
-        # Extract grounding supports
-        grounding_supports = []
-        if getattr(grounding_metadata, "grounding_supports", None):
-            grounding_supports = [
-                {
-                    "segment": {
-                        "start_index": support.segment.start_index
-                        if hasattr(support, "segment")
-                        and hasattr(support.segment, "start_index")
-                        else 0,
-                        "end_index": support.segment.end_index
-                        if hasattr(support, "segment")
-                        and hasattr(support.segment, "end_index")
-                        else 0,
-                        "text": support.segment.text
-                        if hasattr(support, "segment")
-                        and hasattr(support.segment, "text")
-                        else "",
-                    },
-                    "grounding_chunk_indices": list(
-                        support.grounding_chunk_indices or []
-                    ),
-                }
-                for support in grounding_metadata.grounding_supports
-                if hasattr(support, "segment")
-            ]
-
-        # Extract search entry point
-        search_entry_point = {}
-        if getattr(grounding_metadata, "search_entry_point", None):
-            entry_point = grounding_metadata.search_entry_point
-            if hasattr(entry_point, "rendered_content"):
-                search_entry_point = {"rendered_content": entry_point.rendered_content}
-
-        return GroundingBlock(
-            search_queries=search_queries,
-            grounding_chunks=grounding_chunks,
-            grounding_supports=grounding_supports,
-            search_entry_point=search_entry_point,
-            metadata={"grounding_metadata": grounding_metadata},
-        )
-
     def _extract_text_content(self, content: Any) -> str:
         """Extract text content from response, handling various formats.
 
@@ -861,7 +756,7 @@ class GeminiClient(Client):
             ValueError: If no text content found in response.
         """
         if isinstance(content, list):
-            # Filter to only TextBlock content, ignoring GroundingBlock and other types
+            # Filter to only TextBlock content, ignoring other types
             text_blocks = [item for item in content if isinstance(item, TextBlock)]
             if len(text_blocks) == 1:
                 content = text_blocks[0]
