@@ -4,11 +4,25 @@ import base64
 import json
 import os
 import time
-from collections.abc import AsyncGenerator, Sequence
+from collections.abc import AsyncGenerator, Iterable, Sequence
 from dataclasses import InitVar, dataclass, field
 from typing import Any, TypeVar, Unpack
 
 from openai import AsyncOpenAI
+from openai.types import CompletionUsage, ReasoningEffort
+from openai.types.chat import (
+    ChatCompletionAssistantMessageParam,
+    ChatCompletionContentPartImageParam,
+    ChatCompletionContentPartTextParam,
+    ChatCompletionFunctionToolParam,
+    ChatCompletionMessageCustomToolCall,
+    ChatCompletionMessageFunctionToolCall,
+    ChatCompletionMessageFunctionToolCallParam,
+    ChatCompletionMessageParam,
+    ChatCompletionToolMessageParam,
+    ChatCompletionUserMessageParam,
+)
+from openai.types.chat.chat_completion_chunk import ChoiceDeltaToolCall
 from pydantic import BaseModel
 
 from flexai.llm.client import AgentRunArgs, Client, PartialAgentRunArgs, with_defaults
@@ -26,7 +40,9 @@ from flexai.message import (
 from flexai.tool import Tool, ToolType
 
 
-def get_tool_call(tool_use) -> ToolCall:
+def get_tool_call(
+    tool_use: ChatCompletionMessageFunctionToolCall | ChoiceDeltaToolCall,
+) -> ToolCall | None:
     """Get the tool call from a tool use block.
 
     Args:
@@ -35,6 +51,13 @@ def get_tool_call(tool_use) -> ToolCall:
     Returns:
         The tool call from the tool use block.
     """
+    if (
+        not tool_use.id
+        or not tool_use.function
+        or not tool_use.function.name
+        or not tool_use.function.arguments
+    ):
+        return None
     return ToolCall(
         id=tool_use.id,
         name=tool_use.function.name,
@@ -42,7 +65,7 @@ def get_tool_call(tool_use) -> ToolCall:
     )
 
 
-def get_usage_block(usage) -> Usage:
+def get_usage_block(usage: CompletionUsage | None) -> Usage:
     """Extract usage information from the OpenAI response.
 
     Args:
@@ -51,9 +74,13 @@ def get_usage_block(usage) -> Usage:
     Returns:
         A Usage object containing input tokens, output tokens, and generation time.
     """
-    return Usage(
-        input_tokens=usage.prompt_tokens,
-        output_tokens=usage.completion_tokens,
+    return (
+        Usage(
+            input_tokens=usage.prompt_tokens,
+            output_tokens=usage.completion_tokens,
+        )
+        if usage is not None
+        else Usage(input_tokens=0, output_tokens=0)
     )
 
 
@@ -92,6 +119,8 @@ class OpenAIClient(Client):
         defaults = with_defaults(**run_args)
         if "temperature" not in run_args:
             defaults["temperature"] = 1.0
+        if run_args.get("thinking_budget") is None:
+            defaults["thinking_budget"] = self.default_thinking_budget
         return defaults
 
     async def get_chat_response(
@@ -119,21 +148,25 @@ class OpenAIClient(Client):
             messages=messages,
             system=system,
             tools=tools,
-            stream=False,
             **run_args,
         )
-        start = time.time()
-        response = await self.client.chat.completions.create(**params)
-        generation_time = time.time() - start
+        start = time.monotonic()
+        response = await self.client.chat.completions.create(**params, stream=False)
+        generation_time = time.monotonic() - start
 
         # Parse out the tool uses from the response.
         message = response.choices[0].message
-        tool_uses = [get_tool_call(message) for message in (message.tool_calls or [])]
+        tool_uses = [
+            tool_call
+            for tool_use_block in (message.tool_calls or [])
+            if not isinstance(tool_use_block, ChatCompletionMessageCustomToolCall)
+            if (tool_call := get_tool_call(tool_use_block)) is not None
+        ]
         usage = get_usage_block(response.usage)
         usage.generation_time = generation_time
 
         # Get the content to return.
-        content_to_return = tool_uses or message.content
+        content_to_return = tool_uses or message.content or []
         return AIMessage(
             content=content_to_return,
             usage=usage,
@@ -153,9 +186,9 @@ class OpenAIClient(Client):
                 messages=messages,
                 system=system,
                 tools=tools,
-                stream=True,
                 **run_args,
-            )
+            ),
+            stream=True,
         )
         current_tool_call: ToolCall | None = None
         current_text_block: TextBlock | None = None
@@ -172,11 +205,17 @@ class OpenAIClient(Client):
                 current_text_block = current_text_block.append(content)
 
             for tool_call in chunk.choices[0].delta.tool_calls or []:
+                if (
+                    tool_call.function is None
+                    or tool_call.function.name is None
+                    or tool_call.id is None
+                ):
+                    continue
                 if not current_tool_call:
                     current_tool_call = ToolCall(
                         id=tool_call.id,
                         name=tool_call.function.name,
-                        input="",
+                        input=tool_call.function.arguments or "",
                     )
                     yield current_tool_call
 
@@ -202,7 +241,6 @@ class OpenAIClient(Client):
         messages: list[Message],
         system: str | SystemMessage,
         tools: Sequence[Tool] | None,
-        stream: bool,
         **run_args: Unpack[AgentRunArgs],
     ) -> dict:
         """Get the common params to send to the model.
@@ -211,9 +249,8 @@ class OpenAIClient(Client):
             messages: The messages to send to the model.
             system: The system message to send to the model.
             tools: The tools to send to the model.
-            stream: Whether to stream the response.
             **run_args: Additional arguments including temperature, force_tool,
-                allow_tool, stream and other run parameters.
+                allow_tool and other run parameters.
 
         Returns:
             The common params to send to the model.
@@ -223,10 +260,21 @@ class OpenAIClient(Client):
 
         if isinstance(system, str):
             system = SystemMessage(system)
+        thinking_budget = run_args.get("thinking_budget")
+        reasoning_effort: ReasoningEffort | None = (
+            None
+            if thinking_budget is None
+            else "low"
+            if thinking_budget <= 10000
+            else "medium"
+            if thinking_budget <= 20000
+            else "high"
+        )
         kwargs = {
             "model": self.model,
             "messages": self._format_content([system, *messages]),
             "temperature": temperature,
+            "reasoning_effort": reasoning_effort,
         }
 
         # If tools are provided, force the model to use them (for now).
@@ -235,9 +283,6 @@ class OpenAIClient(Client):
             kwargs["tool_choice"] = "required"
             kwargs["parallel_tool_calls"] = False
 
-        if stream:
-            kwargs["stream"] = True
-
         return kwargs
 
     async def _get_structured_response(
@@ -263,16 +308,16 @@ class OpenAIClient(Client):
         Raises:
             ValueError: If the response fails to parse
         """
+        params = self._get_params(
+            messages=messages,
+            system=system,
+            tools=tools,
+            **run_args,
+        )
+        params.pop("reasoning_effort", None)
         # Send the messages to the model and get the response.
-        response = await self.client.beta.chat.completions.parse(
-            **self._get_params(
-                messages=messages,
-                system=system,
-                tools=tools,
-                stream=False,
-                **run_args,
-            ),
-            response_format=model_to_use,
+        response = await self.client.chat.completions.parse(
+            **params, response_format=model_to_use
         )
         result = response.choices[0].message.parsed
         if result is None:
@@ -295,7 +340,7 @@ class OpenAIClient(Client):
         }
 
     @staticmethod
-    def format_tool(tool: Tool) -> dict:
+    def format_tool(tool: Tool) -> ChatCompletionFunctionToolParam:
         """Convert the tool to a description.
 
         Args:
@@ -313,45 +358,97 @@ class OpenAIClient(Client):
                 param_type
             )
 
-        return {
-            "type": "function",
-            "function": {
+        return ChatCompletionFunctionToolParam(
+            type="function",
+            function={
                 "name": tool.name,
                 "description": tool.description,
                 "parameters": input_schema,
             },
-        }
+        )
 
     @classmethod
-    def _format_message_content(
-        cls,
-        contents: Sequence[MessageContent],
-    ) -> list[dict[str, Any]]:
+    def _format_tool_result_image(
+        cls, tool_result: ToolResult
+    ) -> list[ChatCompletionToolMessageParam | ChatCompletionUserMessageParam]:
         return [
-            {"type": "text", "text": content.text}
+            cls._format_tool_result(
+                ToolResult(
+                    tool_call_id=tool_result.tool_call_id,
+                    result="Result was an image, included in the following message.",
+                )
+            ),
+            ChatCompletionUserMessageParam(
+                role="user",
+                content=[
+                    cls._format_text_block(
+                        TextBlock(
+                            text=f"This is the result of the tool call: {tool_result.tool_call_id}"
+                        )
+                    ),
+                    cls._format_image_block(tool_result.result),
+                ],
+            ),
+        ]
+
+    @classmethod
+    def _format_tool_result(
+        cls, tool_result: ToolResult
+    ) -> ChatCompletionToolMessageParam:
+        return ChatCompletionToolMessageParam(
+            role="tool",
+            tool_call_id=tool_result.tool_call_id,
+            content=str(tool_result.result),
+        )
+
+    @classmethod
+    def _format_tool_call(
+        cls, tool_call: ToolCall
+    ) -> ChatCompletionAssistantMessageParam:
+        return ChatCompletionAssistantMessageParam(
+            role="assistant",
+            tool_calls=[
+                ChatCompletionMessageFunctionToolCallParam(
+                    type="function",
+                    id=tool_call.id,
+                    function={
+                        "name": tool_call.name,
+                        "arguments": json.dumps(tool_call.input),
+                    },
+                )
+            ],
+        )
+
+    @classmethod
+    def _format_text_block(
+        cls, text_block: TextBlock
+    ) -> ChatCompletionContentPartTextParam:
+        return ChatCompletionContentPartTextParam(
+            type="text",
+            text=text_block.text,
+        )
+
+    @classmethod
+    def _format_image_block(
+        cls, image_block: ImageBlock
+    ) -> ChatCompletionContentPartImageParam:
+        return ChatCompletionContentPartImageParam(
+            type="image_url",
+            image_url={
+                "url": f"data:{image_block.mime_type};base64,{base64.b64encode(image_block.image).decode('utf-8')}",
+            },
+        )
+
+    @classmethod
+    def _format_message_content_no_tool(
+        cls,
+        contents: Sequence[TextBlock | ImageBlock],
+    ) -> list[ChatCompletionContentPartTextParam | ChatCompletionContentPartImageParam]:
+        return [
+            cls._format_text_block(content)
             if isinstance(content, TextBlock)
-            else {
-                "type": "image_url",
-                "image_url": {
-                    "url": f"data:{content.mime_type};base64,{base64.b64encode(content.image).decode('utf-8')}",
-                },
-            }
+            else cls._format_image_block(content)
             if isinstance(content, ImageBlock)
-            else {
-                "type": "function",
-                "id": content.id,
-                "function": {
-                    "name": content.name,
-                    "arguments": json.dumps(content.input),
-                },
-            }
-            if isinstance(content, ToolCall)
-            else {
-                "role": "tool",
-                "tool_call_id": content.tool_call_id,
-                "content": cls._format_message_content([content.result]),
-            }
-            if isinstance(content, ToolResult)
             else (
                 (_ for _ in ()).throw(
                     TypeError(
@@ -363,70 +460,120 @@ class OpenAIClient(Client):
         ]
 
     @classmethod
-    def _format_content(
+    def _format_message_content(
         cls,
-        value: Message | Sequence[Message],
-        allow_tool: bool = True,
-    ) -> list[dict[str, Any]]:
+        contents: Sequence[TextBlock | ImageBlock | ToolCall | ToolResult],
+    ):
+        return [
+            cls._format_tool_result(content)
+            if isinstance(content, ToolResult)
+            else cls._format_tool_call(content)
+            if isinstance(content, ToolCall)
+            else cls._format_message_content_no_tool([content])[0]
+            for content in contents
+        ]
+
+    @classmethod
+    def _format_content(
+        cls, value: Message | Sequence[Message]
+    ) -> Iterable[ChatCompletionMessageParam]:
         """Format the message content for the OpenAI API.
 
         Args:
             value: The value to format.
-            allow_tool: unused.
 
         Returns:
             The formatted message content.
 
         Raises:
             ValueError: If the message content type is unknown.
+            TypeError: If an ImageBlock is sent as part of a user message.
         """
-        # Anthropic message format.
         if isinstance(value, Message):
             content = value.normalize().content
             if value.role == "user":
                 # Tool calls use a "tool" role instead of "user".
-                messages = []
+                messages: list[ChatCompletionMessageParam] = []
                 # OpenAI wants ToolResults, which currently are simply items in value.content, to be their own messages
                 # Following is logic to extract those into their own messages, and then format the remainder of value.content as a single entity
-                result_content = []
+                result_content: list[ImageBlock | TextBlock] = []
                 for item in content:
                     if isinstance(item, ToolResult):
-                        messages.extend(cls._format_message_content([item]))
+                        if isinstance(item.result, ImageBlock):
+                            messages.extend(cls._format_tool_result_image(item))
+                        else:
+                            messages.append(cls._format_tool_result(item))
+                    elif isinstance(item, ToolCall):
+                        messages.append(cls._format_tool_call(item))
                     else:
                         result_content.append(item)
                 if result_content:
                     messages.append(
-                        {
-                            "role": value.role,
-                            "content": cls._format_message_content(result_content),
-                        }
+                        ChatCompletionUserMessageParam(
+                            role=value.role,
+                            content=cls._format_message_content_no_tool(result_content),
+                        )
                     )
                 return messages
 
-            tool_calls = []
-            non_tool_calls = []
+            messages: list[ChatCompletionMessageParam] = []
             for item in content:
                 if isinstance(item, ToolCall):
-                    tool_calls.append(item)
+                    messages.append(cls._format_tool_call(item))
+                elif isinstance(item, ToolResult):
+                    if isinstance(item.result, ImageBlock):
+                        messages.extend(cls._format_tool_result_image(item))
+                    else:
+                        messages.append(cls._format_tool_result(item))
                 else:
-                    non_tool_calls.append(item)
+                    if value.role == "user":
+                        messages.append(
+                            ChatCompletionUserMessageParam(
+                                role=value.role,
+                                content=cls._format_message_content_no_tool([item]),
+                            )
+                        )
+                        continue
+                    if isinstance(item, ImageBlock):
+                        raise TypeError(
+                            f"ImageBlocks cannot be sent as part of {value.role} messages."
+                        )
+                    if value.role not in ("assistant", "system", "developer"):
+                        raise ValueError(
+                            f"Unknown message role: {value.role}. Only 'user', 'assistant', 'system', and 'developer' are supported."
+                        )
+                    # This looks a bit silly, but pyright wouldn't help us otherwise.
+                    match value.role:
+                        case "system":
+                            messages.append(
+                                {
+                                    "role": value.role,
+                                    "content": [cls._format_text_block(item)],
+                                }
+                            )
+                        case "assistant":
+                            messages.append(
+                                {
+                                    "role": value.role,
+                                    "content": [cls._format_text_block(item)],
+                                }
+                            )
+                        case "developer":
+                            messages.append(
+                                {
+                                    "role": value.role,
+                                    "content": [cls._format_text_block(item)],
+                                }
+                            )
 
-            final_message: dict[str, Any] = {
-                "role": value.role,
-            }
-
-            if non_tool_calls:
-                final_message["content"] = cls._format_message_content(non_tool_calls)
-
-            if tool_calls:
-                final_message["tool_calls"] = cls._format_message_content(tool_calls)
-
-            return [final_message]
+            return messages
 
         # If it's a list of messages, format them.
         if isinstance(value, Sequence) and not isinstance(value, str):
             return [
-                cls._format_content(item, allow_tool=allow_tool)[0] for item in value
+                formatted_message
+                for item in value
+                for formatted_message in cls._format_content(item)
             ]
 
         raise ValueError(f"Unknown message content type: {type(value)}")
