@@ -75,10 +75,7 @@ _OTHER_CONTENT_co = TypeVar(
 
 def normalize_content(
     content: Sequence[MessageContent] | str,
-) -> tuple[
-    TextBlock | ImageBlock | ToolCall | ToolResult | URLContextBlock | GroundingBlock,
-    ...,
-]:
+) -> tuple[TextBlock | ImageBlock | ToolCall | ToolResult, ...]:
     """Normalize the content to a tuple of TextBlock or ImageBlock.
 
     Args:
@@ -97,13 +94,7 @@ def normalize_content(
                         (content,)
                         if isinstance(
                             content,
-                            (
-                                TextBlock,
-                                ImageBlock,
-                                ToolCall,
-                                ToolResult,
-                                URLContextBlock,
-                            ),
+                            (TextBlock, ImageBlock, ToolCall, ToolResult),
                         )
                         else ()
                     )
@@ -165,17 +156,7 @@ class Message(Generic[_CONTENT_co]):
 
     def normalize(
         self,
-    ) -> Message[
-        tuple[
-            ImageBlock
-            | TextBlock
-            | ToolCall
-            | ToolResult
-            | URLContextBlock
-            | GroundingBlock,
-            ...,
-        ]
-    ]:
+    ) -> Message[tuple[ImageBlock | TextBlock | ToolCall | ToolResult, ...]]:
         """Normalize the message content.
 
         Returns:
@@ -185,9 +166,7 @@ class Message(Generic[_CONTENT_co]):
 
     def exclude_image_blocks(
         self,
-    ) -> Message[
-        tuple[TextBlock | ToolCall | ToolResult | URLContextBlock | GroundingBlock, ...]
-    ]:
+    ) -> Message[tuple[TextBlock | ToolCall | ToolResult, ...]]:
         """Exclude image blocks from the message content.
 
         Returns:
@@ -210,6 +189,13 @@ class SystemMessage(Message[_CONTENT_co]):
 
     role: str = field(init=False, default="system")
 
+    def __str__(self) -> str:
+        if isinstance(self.content, str):
+            return self.content
+        return "\n".join(
+            content.text for content in self.content if isinstance(content, TextBlock)
+        )
+
 
 @dataclass
 class UserMessage(Message[_CONTENT_co]):
@@ -395,7 +381,7 @@ class ToolResult(MessageContent):
     tool_call_id: str
 
     # The result of the tool invocation.
-    result: MessageContent
+    result: Any
 
     # The execution time of the tool invocation.
     execution_time: float = 0.0
@@ -404,37 +390,6 @@ class ToolResult(MessageContent):
     is_error: bool = False
 
 
-@dataclass(frozen=True)
-class URLContextBlock(MessageContent):
-    """A block containing URL context metadata from web content access."""
-
-    # The URLs that were accessed
-    urls_accessed: Sequence[str] = field(default_factory=list)
-
-    # Additional metadata from the URL context tool
-    metadata: Mapping[str, Any] = field(default_factory=dict)
-
-
-@dataclass(frozen=True)
-class GroundingBlock(MessageContent):
-    """A block containing Google Search grounding metadata."""
-
-    # The search queries that were executed
-    search_queries: Sequence[str] = field(default_factory=list)
-
-    # The grounding chunks with source information
-    grounding_chunks: Sequence[Mapping[str, Any]] = field(default_factory=list)
-
-    # The grounding supports mapping content to sources
-    grounding_supports: Sequence[Mapping[str, Any]] = field(default_factory=list)
-
-    # Search entry point for widget rendering
-    search_entry_point: Mapping[str, Any] = field(default_factory=dict)
-
-    # Additional grounding metadata
-    metadata: Mapping[str, Any] = field(default_factory=dict)
-
-
 @dataclass(kw_only=True)
 class Usage:
     """Track resource usage for a message."""
