@@ -8,14 +8,12 @@ This file demonstrates how to reuse the generic test utilities for a new provide
 
 import os
 
-# Import test constants
-import sys
-
 import pytest
 
 from flexai import Agent, UserMessage
 from flexai.llm.anthropic import AnthropicClient
-from flexai.message import AIMessage, TextBlock
+from flexai.message import AIMessage, ImageBlock, TextBlock
+from tests.constants import AnthropicModels
 
 # Import shared utilities
 from .utils import (
@@ -41,9 +39,6 @@ from .utils import (
     tool_calling_template,
 )
 
-sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
-from tests.constants import AnthropicModels
-
 # =============================================================================
 # ANTHROPIC-SPECIFIC CONFIGURATIONS
 # =============================================================================
@@ -208,3 +203,114 @@ class TestAnthropicIntegration:
     async def test_multi_turn_conversation(self, anthropic_client):
         """Test maintaining conversation state across multiple turns."""
         await multi_turn_template(anthropic_client)
+
+    def test_format_tool_result_with_dict(self):
+        """Test _format_tool_result with dictionary data."""
+        test_dict = {"key": "value", "number": 42, "nested": {"inner": "data"}}
+        result = AnthropicClient._format_tool_result(test_dict)
+        assert result == str(test_dict)
+
+    def test_format_tool_result_with_list(self):
+        """Test _format_tool_result with list data."""
+        test_list = [1, 2, 3, "string", {"key": "value"}]
+        result = AnthropicClient._format_tool_result(test_list)
+        assert result == str(test_list)
+
+    def test_format_tool_result_with_textblock(self):
+        """Test _format_tool_result with TextBlock."""
+        text_block = TextBlock("Sample text content")
+        result = AnthropicClient._format_tool_result(text_block)
+        expected = [{"type": "text", "text": "Sample text content"}]
+        assert result == expected
+
+    def test_format_tool_result_with_imageblock(self):
+        """Test _format_tool_result with ImageBlock."""
+        # Create a simple 1x1 PNG image in bytes
+        import base64
+
+        simple_png = base64.b64decode(
+            "iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAYAAAAfFcSJAAAADUlEQVR42mNkYPhfDwAChwGA60e6kgAAAABJRU5ErkJggg=="
+        )
+
+        image_block = ImageBlock(image=simple_png, mime_type="image/png")
+        result = AnthropicClient._format_tool_result(image_block)
+
+        assert isinstance(result, list)
+        assert len(result) == 1
+        assert result[0]["type"] == "image"
+        assert result[0]["source"]["type"] == "base64"
+        assert result[0]["source"]["media_type"] == "image/png"
+        assert len(result[0]["source"]["data"]) > 0
+
+    def test_format_tool_result_with_complex_nested_dict(self):
+        """Test _format_tool_result with complex nested dictionary."""
+        complex_dict = {
+            "status": "success",
+            "data": {
+                "users": [
+                    {"id": 1, "name": "Alice", "active": True},
+                    {"id": 2, "name": "Bob", "active": False},
+                ],
+                "metadata": {"total": 2, "page": 1, "filters": ["active", "name"]},
+            },
+            "timestamp": "2024-01-01T00:00:00Z",
+        }
+        result = AnthropicClient._format_tool_result(complex_dict)
+        assert result == str(complex_dict)
+
+    def test_format_tool_result_with_none(self):
+        """Test _format_tool_result with None value."""
+        result = AnthropicClient._format_tool_result(None)
+        assert result == str(None)
+
+    def test_format_tool_result_with_boolean_and_numbers(self):
+        """Test _format_tool_result with boolean and numeric types."""
+        assert AnthropicClient._format_tool_result(True) == str(True)
+        assert AnthropicClient._format_tool_result(False) == str(False)
+        assert AnthropicClient._format_tool_result(42) == str(42)
+        assert AnthropicClient._format_tool_result(3.14) == str(3.14)
+
+    def test_format_tool_result_with_custom_object(self):
+        """Test _format_tool_result with custom object."""
+
+        class CustomObject:
+            def __init__(self):
+                self.value = "test"
+
+            def __str__(self):
+                return f"CustomObject(value={self.value})"
+
+        custom_obj = CustomObject()
+        result = AnthropicClient._format_tool_result(custom_obj)
+        assert result == "CustomObject(value=test)"
+
+    async def test_tool_with_dict_result_integration(self, anthropic_client):
+        """Integration test for tools that return dictionaries."""
+
+        def dict_tool(key: str) -> dict:
+            """A tool that returns a dictionary.
+
+            Args:
+                key: The key to include in the result
+
+            Returns:
+                A dictionary with the provided key and some test data
+            """
+            return {
+                "provided_key": key,
+                "status": "success",
+                "data": {"items": [1, 2, 3], "metadata": {"count": 3}},
+            }
+
+        from flexai.tool import Tool
+
+        tool = Tool.from_function(dict_tool)
+
+        messages = [UserMessage("Use the dict_tool with key 'test_key'")]
+        response = await anthropic_client.get_chat_response(messages, tools=[tool])
+
+        CommonAssertions.assert_valid_response(response)
+        # The response should contain tool calls
+        if isinstance(response.content, list):
+            tool_calls = CommonAssertions.assert_contains_tool_calls(response)
+            assert tool_calls[0].name == "dict_tool"
