@@ -603,16 +603,16 @@ class PerformanceHelper:
 
         @functools.wraps(func)
         async def async_wrapper(*args, **kwargs):
-            start = time.time()
+            start = time.monotonic()
             result = await func(*args, **kwargs)
-            duration = time.time() - start
+            duration = time.monotonic() - start
             return result, duration
 
         @functools.wraps(func)
         def sync_wrapper(*args, **kwargs):
-            start = time.time()
+            start = time.monotonic()
             result = func(*args, **kwargs)
-            duration = time.time() - start
+            duration = time.monotonic() - start
             return result, duration
 
         import asyncio
